import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:tim_ui_kit_push_plugin/channel/flutter_push_base_impl.dart';
import 'package:tim_ui_kit_push_plugin/tim_ui_kit_push_plugin.dart';

class GoogleFCMImpl extends FlutterPushBase{

  final FirebaseMessaging googleFCMMessaging = FirebaseMessaging.instance;

  @override
  Future<String?> getToken() async {
    print("TUIKitPush | Dart | getGoogleToken");
    return await FirebaseMessaging.instance.getToken();
  }

  @override
  void init(PushClickAction onClickNotificationFunction) {
    onClickNotification = onClickNotificationFunction;
    setupGoogleInteractedMessage();
  }

  @override
  void requirePermission() async {
    if(googleFCMMessaging.isSupported()){
      print("TUIKitPush | Dart | getGooglePermission");
      NotificationSettings settings = await googleFCMMessaging.requestPermission(
        alert: true,
        announcement: true,
        badge: true,
        carPlay: false,
        criticalAlert: false,
        provisional: false,
        sound: true,
      );

      if (settings.authorizationStatus == AuthorizationStatus.authorized) {
        print('User granted permission');
      } else if (settings.authorizationStatus == AuthorizationStatus.provisional) {
        print('User granted provisional permission');
      } else {
        print('User declined or has not accepted permission');
      }
    }
  }

  /// 监听Google消息
  Future<void> setupGoogleInteractedMessage() async {
    RemoteMessage? initialMessage =
    await FirebaseMessaging.instance.getInitialMessage();

    if (initialMessage != null) {
      _handleGoogleMessage(initialMessage);
    }

    FirebaseMessaging.onMessageOpenedApp.listen(_handleGoogleMessage);
  }

  /// 处理Google消息
  void _handleGoogleMessage(RemoteMessage message) {
    return onClickNotification!(message.data);
  }

  @override
  bool isSupport() {
    googleFCMMessaging.setAutoInitEnabled(true);
    return googleFCMMessaging.isSupported();
  }
}