import 'dart:async';

import 'package:flutter/services.dart';
import 'package:tim_ui_kit_push_plugin/model/appInfo.dart';
import 'package:tim_ui_kit_push_plugin/channel/apple_apns_impl.dart';
import 'package:tim_ui_kit_push_plugin/utils/utils.dart';
import 'dart:io' show Platform;

import 'channel/flutter_push_base_impl.dart';
import 'channel/google_fcm_impl.dart';
import 'channel/huawei_apns_impl.dart';
export 'model/appInfo.dart';

typedef PushClickAction = void Function(Map<String, dynamic> msg);

class TimUiKitPushPlugin {
  static TimUiKitPushPlugin? _instance;
  TimUiKitPushPlugin._internal({bool? isUseGoogleFCM}) {
    _initPlugin(isUseGoogleFCM);
  }
  factory TimUiKitPushPlugin({bool? isUseGoogleFCM}) {
    _instance ??= TimUiKitPushPlugin._internal(isUseGoogleFCM: isUseGoogleFCM);
    return _instance!;
  }

  static const MethodChannel _channel = MethodChannel('tim_ui_kit_push_plugin');

  late final bool isUseGoogleFCM;
  late final FlutterPushBase flutterPush;

  static String deviceToken = "";
  PushClickAction? onClickNotification;
  bool isUseFlutterPlugin = false;

  _initPlugin(isUseGoogleFCMBool) {
    _channel.setMethodCallHandler(_handleMethod);
    isUseGoogleFCM = isUseGoogleFCMBool;
    if(isUseGoogleFCM){
      flutterPush = GoogleFCMImpl();
      if(flutterPush.isSupport()){
        print("TUIKitPush | Dart Plugin | USE_GOOGLE_FCM");
        isUseFlutterPlugin = true;
      }
    }
    if(Platform.isIOS){
      print("TUIKitPush | Dart Plugin | USE_iOS");
      isUseFlutterPlugin = true;
      flutterPush = AppleAPNSImpl();
    }
  }

  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    switch (call.method) {
      case "TIMPushClickAction":
        print("TUIKitPush | Dart Plugin | on_handleMethod | TIMPushClickAction");
        return onClickNotification!(call.arguments.cast<String, dynamic>());
      default:
        throw UnsupportedError("Unrecongnized Event");
    }
  }

  /// 初始化厂商推送
  void init(
      {required PushClickAction pushClickAction,  PushAppInfo? appInfo}) async{

    if(Platform.isAndroid && await _channel.invokeMethod("isEmuiRom")){
      // 因华为需要使用native异步校验，放到这边进行
      print("TUIKitPush | Dart Plugin | USE HUAWEI");
      isUseFlutterPlugin = true;
      flutterPush = HuaweiImpl();
    }

    print("TUIKitPush | DART | INIT, isUseFlutterPlugin: $isUseFlutterPlugin");
    onClickNotification = pushClickAction;

    if(isUseFlutterPlugin){
      flutterPush.init(pushClickAction);
      return;
    }

    if(Platform.isAndroid && appInfo != null){
      await Utils.setAppInfoForChannel(_channel, appInfo);
    }

    _channel.invokeMethod("initPush");
  }

  /// 申请推送权限
  Future<void> requireNotificationPermission() async {
    if(isUseFlutterPlugin){
      flutterPush.requirePermission();
    }
    await _channel.invokeMethod("getNotificationPermission");
    return;
  }

  /// 获取厂商推送 token
  Future<String> getDevicePushToken() async {
    String token = "";
    try {
      if(isUseFlutterPlugin){
        print("TUIKitPush | Dart | getTokenByFlutter");
        token = await flutterPush.getToken() ?? flutterPush.token;
        print("TUIKitPush | Dart | getTokenByFlutter | DeviceToken: $token");
      } else{
        print("TUIKitPush | Dart | getTokenByNative");
        token = await _channel.invokeMethod("getPushToken");
        print("TUIKitPush | Dart | getTokenByNative | DeviceToken: $token");
      }
    } catch (err) {
      print("getDevicePushToken err $err");
    }
    return token;
  }

  /// 获取安卓厂商品牌，当前仅对安卓有效
  static Future<String?> getOtherPushType() async {
    return await _channel.invokeMethod("getDeviceManufacturer");
  }

  /// 从appInfo中找出离线推送证书ID
  static Future<int?> getBuzId(PushAppInfo appInfo) async {
    if(Platform.isIOS){
      return appInfo.apple_buz_id;
    }
    String device = await getOtherPushType() ?? "";
    print("TUIKitPush | Dart | getOtherPushType | device: $device");
    switch (device){
      case 'oppo':
      case 'oneplus':
      case 'realme':
        if(appInfo.oppo_buz_id != null){
          return appInfo.oppo_buz_id;
        }
        break;
      case 'xiaomi':
      case 'mi':
      case 'blackshark':
        if(appInfo.mi_buz_id != null){
          return appInfo.mi_buz_id;
        }
        break;
      case 'huawei':
      case 'honor':
        if(appInfo.hw_buz_id != null){
          return appInfo.hw_buz_id;
        }
        break;
      case 'meizu':
        if(appInfo.mz_buz_id != null){
          return appInfo.mz_buz_id;
        }
        break;
      case 'vivo':
      case 'iqoo':
        if(appInfo.vivo_buz_id != null){
          return appInfo.vivo_buz_id;
        }
        break;
      default:
        if(appInfo.google_buz_id != null){
          return appInfo.google_buz_id;
        }
        break;
    }

    if(await _channel.invokeMethod("isOppoRom") && appInfo.oppo_buz_id != null){
      return appInfo.oppo_buz_id;
    }
    if(await _channel.invokeMethod("isMiuiRom") && appInfo.mi_buz_id != null){
      return appInfo.mi_buz_id;
    }
    if(await _channel.invokeMethod("isEmuiRom") && appInfo.hw_buz_id != null){
      return appInfo.hw_buz_id;
    }
    if(await _channel.invokeMethod("isMeizuRom") && appInfo.mz_buz_id != null){
      return appInfo.mz_buz_id;
    }
    if(await _channel.invokeMethod("isVivoRom") && appInfo.vivo_buz_id != null){
      return appInfo.vivo_buz_id;
    }
    if(appInfo.google_buz_id != null){
      return appInfo.google_buz_id;
    }

    return 0;
  }


  /// 创建Android端推送通知渠道
  void createNotificationChannel(
      {required String channelId,
      required String channelName,
      required String channelDescription}) async{

    if (Platform.isAndroid) {
      _channel.invokeMethod("createNotificationChannel", {
        "channelId": channelId,
        "channelName": channelName,
        "channelDescription": channelDescription
      });
    }
  }
}
